precision highp float;

attribute vec3 position;
attribute vec4 direction;

uniform sampler2D texture0;
uniform sampler2D texture3;

uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;
uniform vec2 viewport;
uniform float arrowTileSize;
uniform mat4 invProjViewModel;
uniform vec4 dataLatLon;
uniform vec3 uncompress; // multiplicator, offset, scaleFactor
uniform vec2 textureRatio;

#define MULTIPLICATOR   uncompress.r
#define OFFSET          uncompress.g
#define SCALEFACTOR     uncompress.b

#define readTexture2(texUnit, coord) ((texture2D(texUnit, coord).ra * MULTIPLICATOR) * SCALEFACTOR) + OFFSET

varying vec2 texCoord0;
varying float speed;
varying float angle;

#define PI 3.14159265359
#define PI_2 (3.14159265359 / 2.0)

float atan2(in float y, in float x)
{
    bool s = (abs(x) > abs(y));
    return mix(PI/2.0 - atan(x,y), atan(y,x), float(s));
}

float signedAngle(vec2 v1, vec2 v2) 
{
    float perpDot = v1.x * v2.y - v1.y * v2.x;
    return atan2(perpDot, dot(v1, v2));
}

vec2 project(vec4 P)
{
    vec2 p = 0.5 + (P.xyz/P.w).xy * 0.5;
    return p * viewport;
}

// Project from the screen space to the world space
vec4 unproject(vec2 p, float z, float w)
{
    vec4 P = vec4( w*((p/viewport)*2.0 - 1.0), z, w);
    return P;
}

vec2 ToGeodetic2D(vec3 positionOnEllipsoid)
{
    vec3 n = normalize(positionOnEllipsoid);
    return vec2(atan2(n.y, n.x), asin(n.z / length(n)));
}

void main() 
{
    vec2 origin = ToGeodetic2D(position);
    vec2 particleLatLon = origin * 180.0 / PI;

    vec3 pos = position;

    mat4 projViewModel = projection * view * model;
    vec4 currentProjected = projViewModel * vec4(pos, 1.0);
    vec2 currentScreen    = project(currentProjected);

    // Compute pixel coordinates in Normalized Device Coordinates space [0, 1].
    vec2 pixelCoord = 0.5 + (currentProjected.xyz/currentProjected.w).xy * 0.5;

    // Check screen limits, the vertex can be outside of the screen, but there are no data to read.
    if (pixelCoord.x < 0.0 || pixelCoord.y < 0.0 || pixelCoord.x > 1.0 || pixelCoord.y > 1.0)
    {
        // Outside vertex, do nothing.
        gl_Position = currentProjected;
        return;
    }

    vec2 coord = (particleLatLon.xy - dataLatLon.xy) / (dataLatLon.zw - dataLatLon.xy);

    coord = clamp(coord, vec2(0.0, 0.0), vec2(1.0, 1.0));

    // Texture in luminance_alpha mode so use (R || G || B) && A
    vec2 d0 = readTexture2(texture3, coord * textureRatio);

    vec2 dataDirection = vec2(0.0, 1.0);

    // m/s to knot/s
    speed = length(d0) * 1.942200124300808;

    angle = signedAngle(dataDirection, normalize(d0));

    // Normalize vector
    vec2 screenY = dataDirection;
    vec2 screenX = vec2(-screenY.y, screenY.x);

    vec3 X, Y, Z;

    Y = normalize(pos);

    X = cross(Y, vec3(0.0, 0.0, -1.0));
    X = cross(Y, vec3(screenY.x, 0.0, -screenY.y));

    // Check if Y == vec3(0.0, 1.0, 0.0), so cross product give a null vector
    if (length(X) == 0.0)
    {
        // Use another axis for this case
        X = normalize(cross(Y, vec3(0.0, 1.0, 0.0)));
    }
    else
    {
        // length is not null, normalize the vector
        X = normalize(X);
    }
    
    Z = normalize(cross(Y, X));

    // Compute the position of the current corner vertex in screen space
    vec2 dirNorm = normalize(direction.xy);

    // Compute the position of the current corner vertex in screen space
    vec2 currentScreen2 = currentScreen + dirNorm * arrowTileSize;

    // Convert the screen space position in world space
    vec4 p0 = unproject(currentScreen2, currentProjected.z, currentProjected.w);
    p0 = invProjViewModel * p0;

    // Compute the distance in world space between the center and the corner of the quad.
    float dist = distance(p0.xyz, pos);

    // Move the current vertex to a corner of the quad with the distance computed before.
    // The goal is to have a quad which have always the same size but in world space and not
    // oriented to the camera.
    vec3 P = pos + (X * dirNorm.x + Z * dirNorm.y) * dist;

    // Compute the texture coordinates.
    texCoord0 = clamp(direction.xy, 0.0, 1.0);

    gl_Position = projViewModel * vec4(P, 1.0);
}
