#version 100

precision highp float;

attribute vec3 position;
attribute vec4 direction;

uniform sampler2D texture0;
uniform sampler2D texture1;
uniform sampler2D texture3;

uniform mat4 projViewModel;
uniform vec2 viewport;
// half diagonal size, square edge = (arrowTileSize * 2) / sqrt(2)
uniform float arrowTileSize; 
uniform mat4 invProjViewModel;
uniform vec4 dataLatLon;
uniform vec3 uncompress; // multiplicator, offset, scaleFactor
uniform vec3 a;
uniform vec3 b;
uniform vec3 c;
uniform vec2 textureRatio;

#define MULTIPLICATOR   uncompress.r
#define OFFSET          uncompress.g
#define SCALEFACTOR     uncompress.b

#define readTexture2(texUnit, coord) ((texture2D(texUnit, coord).ra * MULTIPLICATOR) * SCALEFACTOR) + OFFSET

varying vec2 texCoord0;
varying vec4 tileCoord;
varying float angle;

#define PI 3.14159265359
#define PI_2 (3.14159265359 / 2.0)

float atan2(in float y, in float x)
{
    float s;

    if(abs(x) > abs(y))
    {
        s = atan(y,x);
    }
    else
    {
        s = PI/2.0 - atan(x,y);
    }

    return s;
}

float signedAngle(vec2 v1, vec2 v2) 
{
    float perpDot = v1.x * v2.y - v1.y * v2.x;
    return atan2(perpDot, dot(v1, v2));
}

vec2 project(vec4 P)
{
    vec2 p = 0.5 + (P.xyz/P.w).xy * 0.5;
    return p * viewport;
}

// Project from the screen space to the world space
vec4 unproject(vec2 p, float z, float w)
{
    vec4 P = vec4( w*((p/viewport)*2.0 - 1.0), z, w);
    return P;
}

vec2 ToGeodetic2D(vec3 positionOnEllipsoid)
{
    vec3 n = normalize(positionOnEllipsoid);
    return vec2(atan2(n.y, n.x), asin(n.z / length(n)));
}

void main() 
{
    vec3 pos = normalize(position.x * a + position.y * b + position.z * c);

    vec2 origin = ToGeodetic2D(pos);
    vec2 particleLatLon = origin * 180.0 / PI;

    if (particleLatLon.x > 180.0)
    {
        particleLatLon.x -= 360.0;
    }
    else if (particleLatLon.x < -180.0)
    {
        particleLatLon.x += 360.0;
    }
    
    //mat4 projViewModel = projection * view * model;
    vec4 currentProjected = projViewModel * vec4(pos, 1.0);
    vec2 currentScreen    = project(currentProjected);

    // Compute pixel coordinates in Normalized Device Coordinates space [0, 1].
    vec2 pixelCoord = 0.5 + (currentProjected.xyz/currentProjected.w).xy * 0.5;

    // Check screen limits, the vertex can be outside of the screen, but there are no data to read.
    if (pixelCoord.x < 0.0 || pixelCoord.y < 0.0 || pixelCoord.x > 1.0 || pixelCoord.y > 1.0)
    {
        // Outside vertex, do nothing.
        gl_Position = currentProjected;
        return;
    }

    // Read data vector
    vec2 coord = (particleLatLon.xy - dataLatLon.xy) / (dataLatLon.zw - dataLatLon.xy); // <- BAAAD

    // Texcoord is wrong with Gille's files, we have to invert v coordinate. Ouch !
    if (MULTIPLICATOR > 1.0)
    {
        coord.y = 1.0 - coord.y;
    }

    if (particleLatLon.y < dataLatLon.y ||
        particleLatLon.y > dataLatLon.w ||
        particleLatLon.x < dataLatLon.x ||
        particleLatLon.x > dataLatLon.z)
    {
        // Particle not on the dataset, discard it.
        gl_Position = vec4(-2.0, 0.0, 0.0, 1.0);
        return;
    }
    else
    {
        float mask = texture2D(texture0, coord * textureRatio).r;

        if (mask < 1.0)
        {
            // Outside vertex, do nothing.
            gl_Position = vec4(-2.0, 0.0, 0.0, 1.0);
            return;
        }
    }

    vec2 d0 = readTexture2(texture3, coord * textureRatio);

    vec2 dataDirection = vec2(0.0, 1.0);

    angle = signedAngle(dataDirection, normalize(d0));

    // Normalize vector
    vec2 screenY = dataDirection;
    vec2 screenX = vec2(-screenY.y, screenY.x);

    vec3 X, Y, Z;

    Y = normalize(pos);

    X = cross(Y, vec3(0.0, 0.0, -1.0));
    X = cross(Y, vec3(screenY.x, 0.0, -screenY.y));

    // Check if Y == vec3(0.0, 1.0, 0.0), so cross product give a null vector
    if (length(X) == 0.0)
    {
        // Use another axis for this case
        X = normalize(cross(Y, vec3(0.0, 1.0, 0.0)));
    }
    else
    {
        // length is not null, normalize the vector
        X = normalize(X);
    }

    Z = normalize(cross(Y, X));

    // Compute the position of the current corner vertex in screen space
    vec2 dirNorm = normalize(direction.xy);

    vec2 currentScreen2 = currentScreen + dirNorm * arrowTileSize;

    // Convert the screen space position in world space
    vec4 p0 = unproject(currentScreen2, currentProjected.z, currentProjected.w);
    p0 = invProjViewModel * p0;

    // Compute the distance in world space between the center and the corner of the quad.
    float dist = distance(p0.xyz, pos);

    // Move the current vertex to a corner of the quad with the distance computed before.
    // The goal is to have a quad which have always the same size but in world space and not
    // oriented to the camera.
    vec3 P = pos + (X * dirNorm.x + Z * dirNorm.y) * dist;

    // Compute the texture coordinates.
    texCoord0 = clamp(direction.xy, 0.0, 1.0);

    // m/s to knot/s
    float speed = length(d0.xy) * 1.942200124300808;

    float barbID = (floor(speed / 5.0) + 0.5) * (1.0 / 21.0);//speed / 5.0 / 21;

    float ratioTex1 = 21.0 / 32.0;
    tileCoord = texture2D(texture1, vec2(barbID * ratioTex1, 0.5));

    gl_Position = projViewModel * vec4(P, 1.0);
}
